;----------------------------------------------------------------------------
; Happy Looser!
; by Omar Cornut / Bock|Zoop (c) in February 2001
;----------------------------------------------------------------------------

; TODO: dancing guy arms are buggy on real SMS since I changed
;       a few interrupts thingies. I think the sprite table is
;       not anymore being written in the vblank, but it should.
;       Need to replace the OTIR by a manual, slower loop.

; TODO: add SMSC header.

.org    $0000

; DEFINITIONS ---------------------------------------------------------------
.include sms.asm
;----------------------------------------------------------------------------
; Thanks to buggy TASM, operators are not always welcome here
TILE_FONT       = 0
TILE_FONT_N     = 59
TILE_MISC       = 59  ; TILE_FONT_N
TILE_MISC_N     = 2   ; 2
TILE_VORTEX     = 61  ;
TILE_VORTEX_N   = 144
TILE_GUY        = 0
TILE_GUY_N      = 120 ; (2*4)+(4*[3/4])+2*4*(3*4)
;----------------------------------------------------------------------------
TILE_ARML1      = 24
TILE_ARML2      = 36
TILE_ARML3      = 48
TILE_ARML4      = 60
;----------------------------------------------------------------------------
TILE_ARMR1      = 72
TILE_ARMR2      = 84
TILE_ARMR3      = 96
TILE_ARMR4      = 108
;----------------------------------------------------------------------------

; VARIABLES -----------------------------------------------------------------
; Miscellaneous
VAR_misc        = RAM + 1               ;   1 byte
VAR_frame_cnt   = RAM + 2               ;   1 byte
; PSG engine
VAR_psg_song    = RAM + 3               ;   2 bytes
VAR_psg_song_p  = RAM + 5               ;   2 bytes
VAR_psg_delay   = RAM + 7               ;   1 byte
; Scrolling engine
VAR_scroll_cnt  = RAM + 8               ;   1 byte
VAR_scroll_mode = RAM + 9               ;   1 byte      (0: disable, 1: line per line, 2: blocks)
VAR_scroll_y    = RAM + 10              ;   1 byte
VAR_scroll_x    = RAM + 11              ; 256 bytes     !! 192 bytes
; Text scrolling engine
VAR_text_pos1   = RAM + 267             ;   2 bytes
VAR_text_pos2   = RAM + 269             ;   2 bytes
VAR_text_delay  = RAM + 271             ;   1 byte
; Other
VAR_vortex      = RAM + 272             ;   1 byte
VAR_guy_arm_L   = RAM + 273             ;   1 byte
VAR_guy_arm_R   = RAM + 274             ;   1 byte
VAR_guy_arm_u   = RAM + 275             ;   1 byte
; Sprites
VAR_sprites_y   = RAM + 276             ;  64 bytes     (64 * 1)
VAR_sprites_xn  = RAM + 340             ; 128 bytes     (64 * 2)
;----------------------------------------------------------------------------

; CODE ----------------------------------------------------------------------
                di                      ; disable interrupts
                im      1               ; interrupt mode 1
                jp      startup

.db "SMS POWER"                         ; It fits :)

; Tools ---------------------------------------------------------------------
.org    $0010
vdp_write_de:
                ld      a, e
                out     (VDP_ADDR), a
                ld      a, d
                out     (VDP_ADDR), a
                ret
.org    $0018
vdp_write_addr_de:
                ld      a, e
                out     (VDP_ADDR), a
                ld      a, d
                or      $40
                out     (VDP_ADDR), a
                ret

.org    $0028
vdp_write_addr_hl:
                ld      a, l
                out     (VDP_ADDR), a
                ld      a, h
                or      $40
                out     (VDP_ADDR), a
                ret

; Interrupt -----------------------------------------------------------------
.org    $0038
interrupt:
                di
                exx
                ex      af, af'

                in      a, (VDP_STATUS)         ; Read VDP status (mandatory)
                and     $80
                jp      z, interrupt_hblank
                ;jp      interrupt_vblank
                ;jp      (hl)
; VBlank interrupt handler --------------------------------------------------
interrupt_vblank:

                ld      a, (VAR_frame_cnt)              ;
                inc     a                               ; VAR_frame_cnt++
                ld      (VAR_frame_cnt), a              ;

                xor     a                               ; VAR_scroll_cnt = 0
                ld      (VAR_scroll_cnt), a             ;

                ld      a, (VAR_scroll_y)               ;
                out     (VDP_ADDR), a                   ; Update VScroll
                ld      a, VREG_VSCROLL                 ;
                out     (VDP_ADDR), a                   ;

;interrupt_vblank_x:
;                ld      a, (VAR_scroll_mode)
;                and     $FF
;                jr      nz, interrupt_vblank_x_end
;                ld      a, (VAR_scroll_x)
;                out     (VDP_ADDR), a
;                ld      a, VREG_HSCROLL
;                out     (VDP_ADDR), a
;interrupt_vblank_x_end:

                call    psg_update                      ; Update PSG engine

interrupt_vblank_end:
                ex      af, af'
                exx
                ei
                ret

; Non Maskable Interrupt (NMI) ----------------------------------------------
.org    $0068
nmi:
                push    af                              ;
                ld      a, (VAR_misc)                   ; Invert PAUSE bit
                xor     00000001b                       ; (yet unused)
                ld      (VAR_misc), a                   ;
                pop     af                              ;
                reti                                    ;

; Identification text, for ROM hackers --------------------------------------
.db             "--[ Happy Looser ]-"
.db             "-[ V.1.0, Feb.2001 ]-"
.db             "-[ by Omar Cornut ]--"
;----------------------------------------------------------------------------

; Startup -------------------------------------------------------------------
startup:
                ld      sp, $DFF0               ; Load stack pointer

                ld      hl, REG_MAP_SRAM        ; Disable battery backed RAM
                ld      (hl), $00               ; Set default memory mapping
                inc     hl                      ;
                ld      (hl), 0                 ;
                inc     hl                      ;
                ld      (hl), 1                 ;
                inc     hl                      ;
                ld      (hl), 2                 ;

                ld      hl, RAM                 ; Clear RAM with zero
                ld      (hl), $00               ;
                ld      de, RAM + 1             ;
                ld      bc, RAM_LEN             ;
                ldir                            ;

                call    psg_init                ; Mute PSG
                call    vdp_init                ; Set VDP registers, clear VRAM

                ld      a, 0                    ; Set palette
                ld      b, 32                   ;
                ld      hl, pal_table_bg        ;
                call    vdp_set_pal             ;

                ld      de, $8604               ; Set sprite to use upper tiles
                rst     10h                     ;

                ld      de, $81A2               ; Enable Vertical interrupts
                rst     10h                     ;

                ld      de, $8A00               ; Horizontal Interrupt
                rst     10h                     ; on each line

                ; Interrupts are still not activated there

introduction:
                ld      hl, gfx_font                    ; Load font & misc
                ld      de, VRAM_TILES + 0              ;
                ld      bc, VRAM_TILE_SIZE*(TILE_FONT_N+TILE_MISC_N)  ; (4 * 8 + 2)
                call    vdp_load_data                   ;

                ld      de, $81E2                       ; Enable display
                rst     10h                             ;

                ld      b, 0
                ld      ix, text_intro
intro_loop:
                ld      hl, VRAM_BG_MAP+(2*32*2)
                ld      de, 32*2
                ld      a, b
                and     $FF
                jr      z, intro_loop_l1_end

                push    bc
intro_loop_l1:
                add     hl, de
                djnz    intro_loop_l1
                pop     bc
intro_loop_l1_end:

                push    bc

                rst     28h
                ld      b, 32
intro_loop_l2:
                ld      a, (ix)
                out     (VDP_DATA), a
                inc     ix
                ld      a, $08
                out     (VDP_DATA), a
                djnz    intro_loop_l2

                ld      bc, $FFFF                       ; Produce a delay
                ;ld      bc, 2                          ;
intro_loop_l3:                                          ;
                dec     bc                              ;
                ld      a, b                            ;
                or      c                               ;
                jr      nz, intro_loop_l3               ;

                pop     bc

                inc     b
                ld      a, b
                cp      16                      ; Number of lines in intro.txt
                jr      nz, intro_loop

                ei                              ; Enable interrupts
                ld      b, 180                  ; 
                call    vdp_frame               ; And wait 6 seconds        
                ld      b, 180                  ;         (180 frames)
                call    vdp_frame               ;

                di
                ld      de, $81A2               ; Disable display
                rst     10h                     ;
                call    vdp_clear               ; Clear background

title:
                ld      hl, gfx_happylooser     ; Load tiles for "HAPPY LOOSER"
                ld      de, VRAM_TILES + 0      ;
                ld      bc, 22 * 16 * VRAM_TILE_SIZE
                call    vdp_load_data           ;

                                                ; Blit "HAPPY LOOSER" to screen
                ld      hl, VRAM_BG_MAP + (32-22)+((24-16)*32)
                ld      b, 22                   ; size x
                ld      c, 16                   ; size y
                ld      d, 0                    ; starting tile number
                ld      e, $00                  ; attribute
                call    vdp_bg_putimage         ;

;               exx                             ; Setup HBL interrupt handler
;               ld      hl, interrupt_hblank    ;
;               exx                             ;

;               ld      a, 0                    ; Setup variables
;               ld      (VAR_frame_cnt), a      ; Not necessary as RAM is
;               ....                            ; cleared before

                ld      de, $0206               ; Setup effect
                call    wave_scroll             ;

                ;call    vdp_frame_one
                ld      de, $81E2               ; Enable display
                rst     10h                     ;

                ei                              ; Enable Z80 interrupts

                ; Testing a song
                ;ld      hl, song_intro_3
                ;call    psg_play
                ;jp      $
                ; Jumping to part 2
                ; jp      part_2

                ld      hl, song_intro_1        ; Start introduction music
                call    psg_play                ;

; Logo vertical scroll --------------------------------------------------------
                ld      a, $A0
                ld      b, 64
main_loop_l0:
                inc     a
                ld      (VAR_scroll_y), a
                push    af
                push    bc
                ld      b, 4
                call    vdp_frame
                pop     bc
                pop     af
                djnz    main_loop_l0
;------------------------------------------------------------------------------

                call    psg_stop                ; Stop music
                ld      hl, song_intro_2        ; Start next music
                call    psg_play                ;

                ;jp     $

; Fade to white ---------------------------------------------------------------
                ld      hl, pal_table_red_to_white_fade ; Fade to white
                ld      a, 0                            ; Starting color = 0
                ld      b, 4                            ; Fading 4 colors
                ld      c, 3                            ; Three steps
                ld      d, 4                            ; Set tempo
                call    vdp_fade                        ; Call function
;------------------------------------------------------------------------------

; Recalculate logo ------------------------------------------------------------
                ld      de, $0001                       ; Calculate non
                call    wave_scroll                     ; distorted logo.
                ld      b, 10                           ; Then wait
                call    vdp_frame                       ; 10 frames.
;------------------------------------------------------------------------------

; White palette ---------------------------------------------------------------
                ld      a, 0
                ld      b, 16
                ld      hl, pal_table_fg
                call    vdp_set_pal
                ld      b, 25                                   ; Wait 30 frames
                call    vdp_frame                               ;
;------------------------------------------------------------------------------

; Restore red palette ---------------------------------------------------------
                ld      a, 0                                    ; Restore
                ld      b, 16                                   ; original
                ld      hl, pal_table_bg                        ; red palette
                call    vdp_set_pal                             ;
                ld      b, 110                                  ; Wait 2 secs
                call    vdp_frame                               ;
;------------------------------------------------------------------------------

                ld      hl, song_intro_3        ; Start next music
                call    psg_play                ;

; Effect 1 --------------------------------------------------------------------
                ld      b, 20
main_loop_l1:
                push    bc

                ld      a, 0
                ld      (VAR_scroll_y), a
                ld      de, $0001
                call    wave_scroll
                ld      b, 6
                call    vdp_frame

                ld      a, 2
                ld      (VAR_scroll_y), a
                ld      de, $0002
                call    wave_scroll
                ld      b, 6
                call    vdp_frame

                pop     bc
                djnz    main_loop_l1
;------------------------------------------------------------------------------

                ld      hl, song_intro_4        ; Start next music
                call    psg_play                ;

; Effect 2 --------------------------------------------------------------------
                ld      b, 20
main_loop_l2:
                push    bc

                ld      a, 1
                ld      (VAR_scroll_y), a
                ld      b, 4
                call    vdp_frame
                ld      de, $0104
                call    wave_scroll

                ld      a, 3
                ld      (VAR_scroll_y), a
                ld      b, 4
                call    vdp_frame
                ld      de, $0003
                call    wave_scroll

                ld      a, 2
                ld      (VAR_scroll_y), a
                ld      b, 4
                call    vdp_frame
                ld      de, $0105
                call    wave_scroll

                pop     bc
                djnz    main_loop_l2
;------------------------------------------------------------------------------

                ld      hl, song_intro_fade             ; Start next music
                call    psg_play                        ;

part_2:
; Fade to white ---------------------------------------------------------------
                ld      hl, pal_table_red_to_white_fade ; Fade to white
                ld      a, 0                            ; Starting color = 0
                ld      b, 4                            ; Fading 4 colors
                ld      c, 3                            ; Three steps
                ld      d, 4                            ; Set tempo
                call    vdp_fade                        ; Call function
;------------------------------------------------------------------------------

                di

                call    vdp_clear                       ; Clear background

                ld      hl, gfx_font                    ; Load font & misc
                ld      de, VRAM_TILES + 0              ;
                ld      bc, VRAM_TILE_SIZE*(TILE_FONT_N+TILE_MISC_N)  ; (4 * 8 + 2)
                call    vdp_load_data                   ;

                ld      hl, gfx_vortex                  ; Load vortex
                ld      de, VRAM_TILES+TILE_VORTEX*VRAM_TILE_SIZE
                ld      bc, VRAM_TILE_SIZE*TILE_VORTEX_N)
                call    vdp_load_data                   ;

                ld      hl, gfx_guy_top                   ; Load guy in second
                ld      de, VRAM_TILES+256*VRAM_TILE_SIZE ; half
                ld      bc, VRAM_TILE_SIZE*TILE_GUY_N     ;
                call    vdp_load_data                     ;

                ei
                ld      de, $0001                       ; Reset scrolling
                call    wave_scroll                     ;

                ld      b, 120                          ;
                call    vdp_frame                       ; Wait two seconds
                call    psg_stop                        ; Stop beats

; Fade to blue ----------------------------------------------------------------
                ld      hl, pal_table_white_to_blue_fade; Fade to white
                ld      a, 0                            ; Starting color = 0
                ld      b, 1                            ; Fading 1 color
                ld      c, 3                            ; Three steps
                ld      d, 8                            ; Set tempo
                call    vdp_fade                        ; Call function

                dec     hl                              ; Set color 16
                ld      a, 16                           ; to the blue
                ld      b, 1
                call    vdp_set_pal                     ; background

                ld      de, $8026                       ; Disable H-Interrupts
                rst     10h                             ;
                ld      de, $8AFF                       ;
                rst     10h                             ;

                di

vortex_init:
                ld      a, 0
                ld      (VAR_vortex), a
                call    vortex_update

;guy_init:
                call    guy_init

;                ei

;------------------------------------------------------------------------------

                ld      hl, song_intro_5                ; Start next music
                call    psg_play                        ;

;                di

                ;ld      hl, VRAM_BG_MAP + 30+((24-8)*32) ; Blit "guy" to screen
                ;ld      b, 4                            ; size x
                ;ld      c, 8                            ; size y
                ;ld      d, TILE_GUY                     ; starting tile number
                ;ld      e, $08                          ; attribute
                ;call    vdp_bg_putimage                 ;

                ei
                ld      b, 1
                call    vdp_frame
                di

                ld      hl, VRAM_BG_MAP + (3 * 32 * 2)
                ld      b, 16
                ld      c, 9
                ld      d, TILE_VORTEX
                ld      e, $04|$08
                call    vdp_bg_putimage_flipv

                ld      hl, VRAM_BG_MAP + (3 * 32 * 2) + 32
                ld      b, 16
                ld      c, 9
                ld      d, TILE_VORTEX
                ld      e, $02|$04|$08
                call    vdp_bg_putimage_flipvh

                ld      hl, VRAM_BG_MAP + (12 * 32 * 2)
                ld      b, 16
                ld      c, 9
                ld      d, TILE_VORTEX
                ld      e, $08
                call    vdp_bg_putimage

                ld      hl, VRAM_BG_MAP + (12 * 32 * 2) + 32
                ld      b, 16
                ld      c, 9
                ld      d, TILE_VORTEX
                ld      e, $02|$08
                call    vdp_bg_putimage_fliph

                ld      hl, VRAM_BG_MAP + (0 * 32 * 2)
                ld      b, 32
                ld      d, TILE_MISC + 0
                ld      e, $08
                call    vdp_bg_puttile

                ld      hl, VRAM_BG_MAP + (2 * 32 * 2)
                ld      b, 32
                ld      d, TILE_MISC + 1
                ld      e, $08
                call    vdp_bg_puttile

                ld      hl, VRAM_BG_MAP + (21 * 32 * 2)
                ld      b, 32
                ld      d, TILE_MISC + 1
                ld      e, $0C
                call    vdp_bg_puttile

                ld      hl, VRAM_BG_MAP + (23 * 32 * 2)
                ld      b, 32
                ld      d, TILE_MISC + 0
                ld      e, $0C
                call    vdp_bg_puttile

                ;ei

text_init:
                ld      hl, scroll_1
                ld      (VAR_text_pos1), hl
                ld      hl, scroll_2
                ld      (VAR_text_pos2), hl

text_loop:
                di

                ld      de, VRAM_BG_MAP + (1 * 32 * 2)
                rst     18h
                ld      hl, (VAR_text_pos1)
                ld      b, 32
text_loop_1:
                ld      a, (hl)
                inc     hl
                out     (VDP_DATA), a
                ld      a, $00                  ; attribute
                out     (VDP_DATA), a
                djnz    text_loop_1

                ld      de, VRAM_BG_MAP + (22 * 32 * 2)
                rst     18h
                ld      hl, (VAR_text_pos2)
                ld      b, 32
text_loop_2:
                ld      a, (hl)
                inc     hl
                out     (VDP_DATA), a
                ld      a, $00                  ; attribute
                out     (VDP_DATA), a
                djnz    text_loop_2

                ei
                ld      b, 4
                call    vdp_frame
                di

                ;ld      d, 0
                ;ld      a, (VAR_vortex)
                ;rra
                ;rra
                ;inc     a
                ;inc     a
                ;ld      e, a
                ;call    vortex_scroll

                call    guy_update
                call    vortex_update

                ld      hl, (VAR_text_pos1)
                inc     hl
                ld      (VAR_text_pos1), hl

                ld      hl, (VAR_text_pos2)
                inc     hl
                ld      (VAR_text_pos2), hl

                jp      text_loop

;----------------------------------------------------------------------------
                jp      $                               ; infinite loop
;----------------------------------------------------------------------------

; Horizontal Interrupt handler ----------------------------------------------
interrupt_hblank:
;                in      a, (PORT_VLINE)         ; Read current line
;                ld      (VAR_scroll_cnt), a     ; from the vertical counter

                ld      a, (VAR_scroll_cnt)     ; horizontal scrolling in
                ld      c, a

                and     $C0                     ; Quit handler if we are
                xor     $C0                     ; out of screen
                jr      z, interrupt_hblank_end ; Necessary?
                                                ; FIXME: not a good way..

                ;ld      c, a
                ld      ix, VAR_scroll_x        ; Setup pointer to current
                ld      b, 0                    ; table, and
                add     ix, bc                  ;

                ld      a, c
                inc     a                       ;
                ld      (VAR_scroll_cnt), a     ;

                ld      a, (ix)                 ; Read in HScroll table and
                out     (VDP_ADDR), a           ; update horizontal scrolling

                ld      a, VREG_HSCROLL         ; with the value
                out     (VDP_ADDR), a           ;

interrupt_hblank_end:
                ex      af, af'
                exx
                ei
                ret
;----------------------------------------------------------------------------

; WAVE_SCROLL() -------------------------------------------------------------
; Wave deform the horizontal scrolling table
;----------------------------------------------------------------------------
;  d = starting value
;  e = range
;----------------------------------------------------------------------------
wave_scroll:
                push    bc
                push    de

                ld      ix, VAR_scroll_x
                ld      a, e                            ; A: range left
                sub     d
                ld      b, 192                          ; B: lines left
                ld      c, 0                            ; C: direction

wave_scroll_loop:                                       ; Decrement 'range left'
                dec     a
                jr      nz, wave_scroll_switch_end      ;

wave_scroll_switch:                                     ;
                ld      a, c                            ; Change direction
                xor     $01                             ;
                ld      c, a                            ;
                ld      a, e                            ;
                jp      wave_scroll_dir_end
wave_scroll_switch_end:                                 ;

                bit     0, c                            ;
                jr      z, wave_scroll_dir_right        ; According to direction
wave_scroll_dir_left:                                   ; increment or decrement
                dec     d                               ; current value.
                jp      wave_scroll_dir_end             ;
wave_scroll_dir_right:                                  ;
                inc     d                               ;
wave_scroll_dir_end:                                    ;

                ld      (ix), d
                inc     ix
                djnz    wave_scroll_loop

                pop     de
                pop     bc
                ret
;----------------------------------------------------------------------------

; VORTEX_UPDATE() -----------------------------------------------------------
; Update vortex palette
;----------------------------------------------------------------------------
; no parameters
;----------------------------------------------------------------------------
vortex_update:
                ld      a, (VAR_vortex)
                inc     a
                cp      6
                jr      nz, vortex_update_2
                xor     a

                ;call    guy_update

vortex_update_2:
                ld      (VAR_vortex), a
                ld      hl, pal_table_vortex_1
                and     $FF
                jr      z, vortex_update_4
vortex_update_3:
                inc     hl
                inc     hl
                inc     hl
                inc     hl
                inc     hl
                inc     hl
                dec     a
                jr      nz, vortex_update_3
vortex_update_4:
                ld      a, 23
                ld      b, 6
                call    vdp_set_pal
                ret
;----------------------------------------------------------------------------

; VORTEX_SCROLL() -----------------------------------------------------------
; ..
;----------------------------------------------------------------------------
;  d = starting value
;  e = range
;----------------------------------------------------------------------------
vortex_scroll:
                push    bc
                push    de

                ld      ix, VAR_scroll_x

                ld      b, 3 * 8
vortex_scroll_f1_loop:
                ld      (ix), 0
                inc     ix
                djnz    vortex_scroll_f1_loop

                ld      a, e                            ; A: range left
                sub     d                               ; B: lines left
                ld      b, 144                          ; 144 = 192 - 2 * 3 * 8
                ld      c, 0                            ; C: direction

vortex_scroll_loop:                                     ; Decrement 'range left'
                dec     a
                jr      nz, vortex_scroll_switch_end    ;

vortex_scroll_switch:                                   ;
                ld      a, c                            ; Change direction
                xor     $01                             ;
                ld      c, a                            ;
                ld      a, e                            ;
                jp      vortex_scroll_dir_end
vortex_scroll_switch_end:                               ;

                bit     0, c                            ;
                jr      z, vortex_scroll_dir_right      ; According to direction
vortex_scroll_dir_left:                                 ; increment or decrement
                dec     d                               ; current value.
                jp      vortex_scroll_dir_end           ;
vortex_scroll_dir_right:                                ;
                inc     d                               ;
vortex_scroll_dir_end:                                  ;

                ld      (ix), d
                inc     ix
                djnz    vortex_scroll_loop

                ld      b, 3 * 8
vortex_scroll_f2_loop:
                ld      (ix), 0
                inc     ix
                djnz    vortex_scroll_f2_loop

                pop     de
                pop     bc
                ret
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
; GUY_INIT() ----------------------------------------------------------------
; Create sprites for dancing guy
;----------------------------------------------------------------------------
; no parameters
;----------------------------------------------------------------------------
guy_init:
                xor     a
                ld      (VAR_guy_arm_L), a
                ld      (VAR_guy_arm_R), a

                ld      iy, VAR_sprites_y
                ld      ix, VAR_sprites_xn

                ld      hl, guy_mapping
guy_init_loop:
                ld      a, (hl)
                cp      VRAM_SPR_LAST
                jr      z, guy_init_end
                ld      (iy), a
                inc     hl
                inc     iy
                ld      a, (hl)
                ld      (ix), a
                inc     hl
                inc     ix
                ld      a, (hl)
                ld      (ix), a
                inc     hl
                inc     ix
                jr      guy_init_loop

guy_init_end:
                ; ret
                jp      vdp_sprite_blit
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
; GUY_UPDATE() --------------------------------------------------------------
; Update sprites ofancing guy
;----------------------------------------------------------------------------
; no parameters
;----------------------------------------------------------------------------
guy_update:
                push    af

                call    vdp_sprite_blit         ; blitting sprites at first
                                                ; so it is done inside vblank

guy_update_AL:
                ld      a, (VAR_guy_arm_L)
                inc     a
                and     $3
                ld      (VAR_guy_arm_L), a

                ld      b, a
                ld      a, TILE_ARML1
guy_update_AL_l1:
                add     a, 12
                djnz    guy_update_AL_l1

                ld      ix, VAR_sprites_xn+24           ; 12 * 2
                ld      b, 6
guy_update_AL_l2:
                inc     ix
                ld      (ix), a
                inc     ix
                inc     a
                inc     a
                djnz    guy_update_AL_l2


guy_update_AR:
                ld      a, (VAR_guy_arm_R)
                inc     a
                and     $3
                ld      (VAR_guy_arm_R), a

                ld      b, a
                ld      a, TILE_ARMR1
guy_update_AR_l1:
                add     a, 12
                djnz    guy_update_AR_l1

                ld      ix, VAR_sprites_xn+36           ; 18 * 2
                ld      b, 6
guy_update_AR_l2:
                inc     ix
                ld      (ix), a
                inc     ix
                inc     a
                inc     a
                djnz    guy_update_AR_l2

                ld      a, (VAR_guy_arm_u)              ; Every 16 updates
                inc     a                               ;
                and     $15                             ;
                ld      (VAR_guy_arm_u), a              ;
                jr      nz, guy_update_end              ;

                ld      a, (VAR_guy_arm_R)              ; Unsynchronize arms
                inc     a                               ;
                ld      (VAR_guy_arm_R), a              ;

guy_update_end:
                pop     af
                ret
;----------------------------------------------------------------------------



.include "vdp.asm"
.include "psg.asm"

; DATA ----------------------------------------------------------------------
text_intro:
.include "intro.inc"
scroll_2:
.include "scroll2.inc"
scroll_1:
.include "scroll1.inc"
palettes:
.include "palette.inc"
songs:
.include "songs.inc"
;----------------------------------------------------------------------------

; GRAPHICS ------------------------------------------------------------------
.include "gfx_hl.inc"
;----------------------------------------------------------------------------
.include "gfx_font.inc"  ; keeping the order of those two is mandatory
.include "gfx_misc.inc"  ; for when loading tiles in the program.
;----------------------------------------------------------------------------
.include "gfx_guy.inc"   ; keeping the order of those two is mandatory
.include "gfx_vort.inc"  ; for when loading tiles in the program.
;----------------------------------------------------------------------------

; HEADER / END OF PROGRAM ---------------------------------------------------
.org    $7FF0
.db     "TMR SEGA"
.db     " FOREVER"      ; Checksum
.org    $8000
.end
;----------------------------------------------------------------------------

